import json
from .. import const
from ..templates import templates
from ..package import PSMReader
from ..globals import *
import os

from ..globals import take_input
input = take_input


def addScript(name, command, description=""):
    psm = PSMReader()
    psm.add_script(
        name=name,
        command=command,
        description=description,
    )
    psm.write()
    print('Script added successfully.')


def initialize(template_name, disable_oninit=False):
    template = templates[template_name]
    schema = template['body']
    schema["version"] = '0.0.0'
    schema["config"] = {}
    schema["dependencies"] = {"prod": [], "common": [], "dev": []}
    schema["environment"] = ""
    schema["use_environment"] = True
    current_dir = os.path.basename(os.getcwd())
    schema["name"] = input(
        f'Project name ({current_dir}): ') or current_dir

    schema["description"] = input('Description: ') or ""

    schema["author"] = input('Author: ') or ""
    schema["author_email"] = input('Author e-mail: ') or ""
    schema["url"] = input('Url: ') or ""
    oninit = template.get('oninit', None)
    if oninit and not disable_oninit:
        oninit()
    with open(const.filename, 'wt') as s:
        json.dump(schema, s, indent=4, sort_keys=True)
    print('Successfully initialized ' +
          const.filename+f' with {template_name} template.')


def removeScript(name):
    psm = PSMReader()
    psm.remove_script(name)
    psm.write()
    print('Script removed successfully.')



import sys
import click

from .basic import basic
from .pip import pip
from .environment import environment
from .get import get

commands = [
    basic, 
    pip,
    environment,
    get
]

main = click.CommandCollection(sources=commands) # type: ignore

def cli():
    args = sys.argv
    if "--help" in args or len(args) == 1:
        print("Python Script Manager")
    try:
        main()
    except Exception as e:
        print("❌ "+str(e))



import click
from ..basic.funcs import runScriptDirectly
from ..globals import MultiCommand
from ..package import PSMReader

from ..globals import take_input
input = take_input

@click.group(cls=MultiCommand)
def environment():
    pass


@environment.command("set:env")
def set_env_command(**kwargs):
    """Set environment of project"""
    psm = PSMReader()
    old_env = psm.get_environment()
    new_env = input(
        f'Enter environment name{f" ({old_env})" if old_env else ""}: ')
    psm.set_environment(new_env)
    psm.write()
    print(f'Successfully updated environment: {old_env} -> {new_env}')


@environment.command("set:use-env")
@click.option('--activate/--deactivate',is_flag=True,default=True)
def set_use_env_command(activate,*args, **kwargs):
    """Set environment usage of project"""
    psm = PSMReader()
    psm.set_use_environment(activate)
    psm.write()
    print(
        f'Successfully {"de" if not activate else ""}activated environment usage')



from . import const
import json
import os
import click
from .package import PSMReader
from termcolor2 import c
from PyInquirer import prompt


def loadScripts():
    with open(const.filename, 'rt') as s:
        return json.load(s)['scripts']


def runScriptDirectly(script):
    print('\n\t' + c('>').blue + c('>').yellow + f' {script}\n')
    psm_obj = PSMReader()
    if psm_obj.get_use_environment():
        os.system(f'snakenv {psm_obj.get_environment()} -c "{script}" --not-create')
    else:
        os.system(script)


def runScriptIfExist(name):
    scripts = loadScripts()
    called_script = scripts.get(name, None)
    if called_script:
        cmd = called_script["command"]
        runScriptDirectly(cmd)
    else:
        pass


def runScript(name,other_arguments=""):
    scripts = loadScripts()
    called_script = scripts.get(name, None)
    if called_script:
        cmd = called_script["command"]
        runScriptDirectly(cmd.format(args=other_arguments))
    else:
        raise click.ClickException(f'Can not find script named "{name}"')


class MultiCommand(click.Group):
    def command(self, *args, **kwargs):
        """Behaves the same as `click.Group.command()` except if passed
        a list of names, all after the first will be aliases for the first.
        """
        def decorator(f):
            if isinstance(args[0], list):
                _args = [args[0][0]] + list(args[1:])
                for alias in args[0][1:]:
                    cmd = super(MultiCommand, self).command(
                        alias, *args[1:], **kwargs)(f)
                    cmd.short_help = "Alias for '{}'".format(_args[0])
            else:
                _args = args
            cmd = super(MultiCommand, self).command(
                *_args, **kwargs)(f)
            return cmd
        return decorator

def take_input(text):
    questions = [
        {
            'type':'input',
            'name':'data',
            'message':text
        }
    ]

    answers = prompt(questions)
    return answers['data']


import click
from ..basic.funcs import runScriptDirectly
from ..globals import MultiCommand
from ..package import PSMReader
from typing import List
from ..globals import take_input
input = take_input


@click.group(cls=MultiCommand)
def get():
    pass


@get.command("get")
@click.option('--dev', required=False, is_flag=True)
@click.option('--prod', required=False, is_flag=True)
@click.argument('package_names', required=True, nargs=-1)
def get_command(dev: bool, prod: bool, package_names: List[str]) -> None:
    psm: PSMReader = PSMReader()
    package_scope: str = "common"
    if dev+prod < 2:
        package_scope = "dev" if dev else "prod"
    psm.add_dependency(package_names, package_scope)
    runScriptDirectly(f'pip install {" ".join(package_names)}')
    psm.write()


@get.command("get:deps")
@click.option('--dev', required=False, is_flag=True)
@click.option('--prod', required=False, is_flag=True)
def get_deps_command(dev: bool, prod: bool) -> None:
    psm: PSMReader = PSMReader()
    deps: List[str] = psm.get_dependencies(
        "dev" if dev else ("prod" if prod else "all"))
    runScriptDirectly(f"pip install {' '.join(deps)}")


@get.command("get:load-from")
@click.option('--dev', required=False, is_flag=True)
@click.option('--prod', required=False, is_flag=True)
@click.argument('file_name', default='requirements.txt')
def get_load_from_reqs_command(dev: bool, prod: bool, file_name: str) -> None:
    psm: PSMReader = PSMReader()
    def parse_requirements(filename):
        lineiter = (line.strip() for line in open(filename))
        return [line.split('==')[0] for line in lineiter if line and not line.startswith("#")]
    reqs = parse_requirements(file_name)
    reqs_scope = "common"
    if dev+prod < 2:
        reqs_scope = "dev" if dev else "prod"
    psm.add_dependency(reqs, reqs_scope)
    runScriptDirectly(f"pip install {' '.join(reqs)}")



import click
from .. import const
from ..basic.funcs import runScriptDirectly

@click.group()
def pip():
    pass

@pip.command(name="install")
def install_command(**kwargs):
    """Install dependencies in requirements.txt"""
    runScriptDirectly('pip install -r requirements.txt')


@pip.command(name="freeze")
def freeze_command(**kwargs):
    """Output dependencies of project"""
    runScriptDirectly('pip freeze > requirements.txt')



flaskmng = {
    "body": {
        "scripts":{
            "newapp":{
                "command":"flaskmng startapp",
                "description":"Creating new app for project."
            },
            "rmapp":{
                "command":"flaskmng removeapp",
                "description":"Removing app from project."
            },
            "mkmig":{
                "command":"flask db migrate {args}",
                "description":"Make migrations for project."
            },
            "mig":{
                "command":"flask db upgrade",
                "description":"Upgrade db for project."
            },
            "dbinit":{
                "command":"flask db init",
                "description":"Initialize db for project."
            },
            "start":{
                "command":"flask run",
                "description":"For starting project."
            }
        }
    }
}


import click
from prettytable import PrettyTable
from .funcs import *
from .. import const
from ..globals import MultiCommand, runScriptDirectly
from ..package import PSMReader

from ..globals import take_input
input = take_input

@click.group(cls=MultiCommand)
def basic():
    pass


# Initialize psm
@basic.command("init")
@click.option('-t', '--template', 'template', help="Name of template")
@click.option('--files', is_flag=True, help='Init with file structure (if supported by template)')
@click.option('--disable-oninit', 'disable_oninit', is_flag=True, help='Disable work of oninit event')
def init_command(files, **kwargs):
    """Initialize PSM in current folder"""
    template = kwargs.pop('template') or input(
        'Which template you wanna use? (blank): ') or 'blank'
    initialize(
        template,
        disable_oninit=kwargs.pop('disable_oninit')
    )


# Run script
@basic.command("run", context_settings=dict(
    ignore_unknown_options=True,
    allow_extra_args=True,
))
@click.argument('name', required=True)
@click.argument('other_args', type=click.UNPROCESSED, required=False)
@click.pass_context
def run_command(*args, **kwargs):
    """Run PSM script with NAME"""
    ctx = args[0]
    psm_obj = PSMReader()
    
    try:
        other_arguments = (
            ctx.params["other_args"] + f""" "{' '.join(ctx.args)}" """) or ""
    except Exception as e:
        other_arguments = ""
    name = kwargs.pop('name')
    while not name:
        name = input('Enter name of script to run')
    runScript(name, other_arguments=other_arguments)


# Add script
@basic.command("add")
@click.option('-n', '--name', 'name', required=False, help="Name for new script")
@click.option('-c', '--command', 'command', required=False, help="Command that script will used for")
@click.option('-d', '--description', 'description', required=False, help="Description for new script (optional)")
def add_command(**kwargs):
    """Add new script"""
    name = kwargs.pop('name')
    command = kwargs.pop('command')
    description = kwargs.pop('description')
    while not name:
        name = input('Enter name for script: ')
    while not command:
        command = input('Enter command for script: ')
    if not description:
        description = input('Enter description for script (optional): ')
    addScript(name, command, description)


# List scripts
@basic.command("list")
def list_command(**kwargs):
    """Generate list of scripts as table"""
    cmds = PrettyTable()
    cmds.field_names = ['Name', 'Command', 'Description']
    scripts = loadScripts()
    for name, data in scripts.items():
        row = (name, data["command"], data["description"])
        cmds.add_row(row)
    cmds.align = 'l'
    click.echo(cmds)


# Remove script
@basic.command("rm")
@click.option('-n', '--name', 'name', required=False, help="Name of script to remove")
def rm_command(**kwargs):
    """Remove script"""
    name = kwargs.pop('name')
    while not name:
        name = input('Enter name of script to remove: ')
    removeScript(name)


# Special Scripts
@basic.command("build")
def build_command(**kwargs):
    """Special script that will execute "psm run build" """
    runScript('build')


@basic.command("start")
def start_command(**kwargs):
    """Special script that will execute "psm run start" """
    runScript('start')


@basic.command("deploy")
def deploy_command(**kwargs):
    """Special script that will execute "psm run deploy" """
    runScriptIfExist('predeploy')
    runScript('deploy')
    runScriptIfExist('postdeploy')


@basic.command(['set:version', 'set:v'])
def set_version_command(**kwargs):
    """Set version number for your project"""
    from ..package import PSMReader
    psm = PSMReader(const.filename)
    new_version = input(f'Enter new version number ({psm.data["version"]}): ')
    old_version = psm.data["version"]
    if new_version and new_version != old_version:
        psm.data['version'] = new_version
        psm.write()
        print(f'Successfully updated version: {old_version} -> {new_version}')
